<?php

namespace Huge\Rest\Http;

class HttpResponse {

    const DEFAULT_CONTENT_TYPE = 'application/json';
    
    protected $code;
    protected $body;
    private $headers;
    protected $entity;
    
    private static  $STATUS = array(
        100 => 'HTTP/1.1 100 Continue',
        101 => 'HTTP/1.1 101 Switching Protocols',
        200 => 'HTTP/1.1 200 OK',
        201 => 'HTTP/1.1 201 Created',
        202 => 'HTTP/1.1 202 Accepted',
        203 => 'HTTP/1.1 203 Non-Authoritative Information',
        204 => 'HTTP/1.1 204 No Content',
        205 => 'HTTP/1.1 205 Reset Content',
        206 => 'HTTP/1.1 206 Partial Content',
        300 => 'HTTP/1.1 300 Multiple Choices',
        301 => 'HTTP/1.1 301 Moved Permanently',
        302 => 'HTTP/1.1 302 Found',
        303 => 'HTTP/1.1 303 See Other',
        304 => 'HTTP/1.1 304 Not Modified',
        305 => 'HTTP/1.1 305 Use Proxy',
        307 => 'HTTP/1.1 307 Temporary Redirect',
        400 => 'HTTP/1.1 400 Bad Request',
        401 => 'HTTP/1.1 401 Unauthorized',
        402 => 'HTTP/1.1 402 Payment Required',
        403 => 'HTTP/1.1 403 Forbidden',
        404 => 'HTTP/1.1 404 Not Found',
        405 => 'HTTP/1.1 405 Method Not Allowed',
        406 => 'HTTP/1.1 406 Not Acceptable',
        407 => 'HTTP/1.1 407 Proxy Authentication Required',
        408 => 'HTTP/1.1 408 Request Time-out',
        409 => 'HTTP/1.1 409 Conflict',
        410 => 'HTTP/1.1 410 Gone',
        411 => 'HTTP/1.1 411 Length Required',
        412 => 'HTTP/1.1 412 Precondition Failed',
        413 => 'HTTP/1.1 413 Request Entity Too Large',
        414 => 'HTTP/1.1 414 Request-URI Too Large',
        415 => 'HTTP/1.1 415 Unsupported Media Type',
        416 => 'HTTP/1.1 416 Requested Range Not Satisfiable',
        417 => 'HTTP/1.1 417 Expectation Failed',
        500 => 'HTTP/1.1 500 Internal Server Error',
        501 => 'HTTP/1.1 501 Not Implemented',
        502 => 'HTTP/1.1 502 Bad Gateway',
        503 => 'HTTP/1.1 503 Service Unavailable',
        504 => 'HTTP/1.1 504 Gateway Time-out',
        505 => 'HTTP/1.1 505 HTTP Version Not Supported'
    );
    
    public function __construct($status = 200) {
        $this->code = $status;
        $this->body = null;
        $this->headers = array();
        $this->entity = null;
        
        $this->setContentType(self::DEFAULT_CONTENT_TYPE);
    }
    
    public static function ok(){
        return new HttpResponse(200);
    }
    
    public function hasEntity(){
        return $this->entity !== null;
    }
    
    public function getCode() {
        return $this->code;
    }
    
    /**
     * Ajout l'entête pour l'expiration
     * 
     * @param int $seconds
     * @return \Huge\Rest\Http\HttpResponse
     */
    public function expires($seconds = 0){
        $this->addHeader('Expires', gmdate('D, d M Y H:i:s', time() + $seconds) . ' GMT');
        
        return $this;
    }

    public function contentTypeTxt(){
        $this->setContentType('text/plain');
        return $this;
    }
    
     public function contentTypeJson(){
        $this->setContentType('application/json');
        return $this;
    }
    
    public function contentTypeXml(){
        $this->setContentType('application/xml');
        return $this;
    }
    
    /**
     * @deprecated 
     * 
     * @param int $code
     * @return \Huge\Rest\Http\HttpResponse
     */
    public static function code($code) {
        return new HttpResponse($code);
    }
    
    public static function status($code) {
        return new HttpResponse($code);
    }

    public function getBody() {
        return $this->body;
    }

    public function body($body) {
        $this->body = $body;
        return $this;
    }

    public function getHeaders() {
        return $this->headers;
    }

    public function addHeader($name, $value) {
        $this->headers[$name] = $value;
        return $this;
    }

    public function getEntity() {
        return $this->entity;
    }

    public function entity($entity) {
        $this->entity = $entity;
        return $this;
    }
    
    public function setContentType($type){
        $this->headers['Content-Type'] = $type;
        return $this;
    }
    
    public function build($withBody = true){
        foreach($this->headers as $key => $value){
            @header($key.': '.$value, true);
        }
        
        if(isset(self::$STATUS[$this->code])){
            @header(self::$STATUS[$this->code]);
        }
        
        if($withBody && ($this->body !== null)){
            echo $this->body;
        }
    }
}

